precision highp float;


varying vec2 st;
uniform sampler2D inputImageTexture;


uniform float intensity;

mat4  saturationMatrix(float saturation )
{
    vec3 luminance = vec3(0.299, 0.587, 0.114);

    float oneMinusSat = 1.0 - saturation;

    vec3 red = vec3( luminance.x * oneMinusSat );
    red+= vec3( saturation, 0.0, 0.0 );

    vec3 green = vec3( luminance.y * oneMinusSat );
    green += vec3( 0.0, saturation, 0.0 );

    vec3 blue = vec3( luminance.z * oneMinusSat );
    blue += vec3( 0.0, 0.0, saturation );

    return mat4( red,     0.0,
    green,   0.0,
    blue,    0.0,
    0.0, 0.0, 0.0, 1 );
}

//vec3 hsv2rgb(in vec3 c)
//{
//    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
//    return c.z * mix(vec3(1.0), rgb, c.y);
//}


vec3 rgb2hsv(in vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}


void main() {

    vec3 origin = vec3(330./360., 375./360., 420./360.);

    vec3 col =  texture2D(inputImageTexture, st).rgb;

    vec3 hsv = rgb2hsv(col);

    float h = mix(hsv.x + 1., hsv.x, step(origin.z - 1., hsv.x));

    float alpha = min(abs(h - origin.y) * 360. / 60., 1.0);

//    hsv.y = hsv.y * (1.0 + alpha * intensity);

    gl_FragColor = mix(vec4(col, 1.0), saturationMatrix(intensity) * vec4(col, 1.0), alpha);

//    gl_FragColor = vec4(result, 1.0);
}