precision highp float;

#define MAX_CHROME_ERROR 0.2

#define MIN_CHROME_ERROR 0.02

varying vec2 st;
uniform sampler2D inputImageTexture;
uniform float hue;
uniform float tolerance;
uniform float fade;

float rgb2gray(vec3 color){
    return dot(color, vec3(0.299, 0.587, 0.114));
}

vec3 rgb2hsv(vec3 color){
    float min_rgb = min(color.r, min(color.g, color.b));
    float v = max(color.r, max(color.g, color.b));
    float s = 0.;
    float h = 0.;
    if (v > min_rgb){
        s = (v - min_rgb) / v;
        h = 0.;
        if (v == color.r){
            h = (color.g - color.b) / (6. * (v - min_rgb));
        } else if (v == color.g){
            h = (color.b - color.r) / (6. * (v - min_rgb)) + 1. / 3.;
        } else if (v == color.b){
            h = (color.r - color.g) / (6. * (v - min_rgb)) + 2. / 3.;
        }
        if (h < 0.){
            h = h + 1.;
        }
    }
    return vec3(h, s, v);
}


void main() {
    vec4 color = texture2D(inputImageTexture, st);

    vec4 gray = mix(color, vec4(vec3(rgb2gray(color.rgb)), color.a), fade);

    vec3 hsv = rgb2hsv(color.rgb);

    if(abs(hsv.x - hue) > 0.5){
        hsv.x = mix(hsv.x + 1., hsv.x - 1., step(0.5, hsv.x));
    }

    float beta = min(abs(hsv.x - hue), (MAX_CHROME_ERROR * tolerance +  MIN_CHROME_ERROR))
    / (MAX_CHROME_ERROR * tolerance + MIN_CHROME_ERROR);

    beta = pow(beta, 4.);

    color = mix(color, gray, beta);

    gl_FragColor = color;
}